// -*- C++ -*-
/*!
 * @file  CameraPositionCorrectionRTC.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "CameraPositionCorrectionRTC.h"

// Module specification
// <rtc-template block="module_spec">
static const char* camerapositioncorrectionrtc_spec[] =
  {
    "implementation_id", "CameraPositionCorrectionRTC",
    "type_name",         "CameraPositionCorrectionRTC",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "Saitama Univ. Design Lab.",
    "category",          "EngineeringSample",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.CalibrationX", "320",
    "conf.default.CalibrationY", "240",
    "conf.default.LengthX", "500.0",
    "conf.default.LengthY", "400.0",
    // Widget
    "conf.__widget__.CalibrationX", "text",
    "conf.__widget__.CalibrationY", "text",
    "conf.__widget__.LengthX", "text",
    "conf.__widget__.LengthY", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
CameraPositionCorrectionRTC::CameraPositionCorrectionRTC(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_positionIn("position", m_position),
    m_displacementOut("displacement", m_displacement),
    m_triggerPort("trigger")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
CameraPositionCorrectionRTC::~CameraPositionCorrectionRTC()
{
}



RTC::ReturnCode_t CameraPositionCorrectionRTC::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("position", m_positionIn);
  
  // Set OutPort buffer
  addOutPort("displacement", m_displacementOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_triggerPort.registerConsumer("recogPort", "RecognitionService", m_recogPort);
  
  // Set CORBA Service Ports
  addPort(m_triggerPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("CalibrationX", m_CalibrationX, "320");
  bindParameter("CalibrationY", m_CalibrationY, "240");
  bindParameter("LengthX", m_LengthX, "500.0");
  bindParameter("LengthY", m_LengthY, "400.0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraPositionCorrectionRTC::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraPositionCorrectionRTC::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraPositionCorrectionRTC::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CameraPositionCorrectionRTC::onActivated(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"           Activate           "<<std::endl<<"******************************"<<std::endl<<std::endl;

	std::cout<<"Configuration"<<std::endl<<"[Nʒu"<<std::endl<<" CalibrationX : "<<m_CalibrationX<<" [pixel]"<<std::endl<<" CalibrationY : "<<m_CalibrationY<<" [pixel]"<<std::endl<<"ʒ"<<std::endl<<" m_LengthX : "<<m_LengthX<<" [mm]"<<std::endl<<" m_LengthY : "<<m_LengthY<<" [mm]"<<std::endl<<std::endl;

	std::cout<<std::endl<<"******************************"<<std::endl<<"           Execute            "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t CameraPositionCorrectionRTC::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"          Deactivate          "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t CameraPositionCorrectionRTC::onExecute(RTC::UniqueId ec_id)
{
	if(m_positionIn.isNew()){//XV
		m_positionIn.read();//ǂݍ
		m_displacement.data.x = (double)(m_position.data[11]-m_CalibrationX)/640.0*m_LengthX/1000.0;//pixel->m
		m_displacement.data.y = (double)(m_position.data[15]-m_CalibrationY)/480.0*m_LengthY/1000.0;//pixel->m
		m_displacement.data.z = 0.0;//{RTCłZ͍lȂ

		//\
		std::cout<<"Position"<<std::endl<<" X : "<<m_position.data[11]<<" [pixel]"<<std::endl<<" Y : "<<m_position.data[15]<<" [pixel]"<<std::endl;
		std::cout<<"Displacement"<<std::endl<<" X : "<<m_displacement.data.x<<" [m]"<<std::endl<<" Y : "<<m_displacement.data.y<<" [m]"<<std::endl<<" Z : "<<m_displacement.data.z<<" [m]"<<std::endl<<std::endl;

		m_displacementOut.write();//
	}

  return RTC::RTC_OK;
}


RTC::ReturnCode_t CameraPositionCorrectionRTC::onAborting(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"            Error             "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraPositionCorrectionRTC::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CameraPositionCorrectionRTC::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraPositionCorrectionRTC::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraPositionCorrectionRTC::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CameraPositionCorrectionRTCInit(RTC::Manager* manager)
  {
    coil::Properties profile(camerapositioncorrectionrtc_spec);
    manager->registerFactory(profile,
                             RTC::Create<CameraPositionCorrectionRTC>,
                             RTC::Delete<CameraPositionCorrectionRTC>);
  }
  
};


