// -*- C++ -*-
/*!
 * @file  IntegratedControlRTC.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */
//#define RECTANGULAR //2W^
//#define HORIZONTAL //4֐ߌ^
#define VERTICAL //6֐ߌ^

#include "IntegratedControlRTC.h"

// Module specification
// <rtc-template block="module_spec">
static const char* integratedcontrolrtc_spec[] =
  {
    "implementation_id", "IntegratedControlRTC",
    "type_name",         "IntegratedControlRTC",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "Saitaam Univ. Design Lab.",
    "category",          "EngineeringSample",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.teachingData", "PointData.txt",
    "conf.default.XDisplacementLimitMax", "50.0",
    "conf.default.XDisplacementLimitMin", "-100.0",
    "conf.default.YDisplacementLimitMax", "10",
    "conf.default.YDisplacementLimitMin", "-10",
    "conf.default.cycleCount", "0",
    // Widget
    "conf.__widget__.teachingData", "text",
    "conf.__widget__.XDisplacementLimitMax", "text",
    "conf.__widget__.XDisplacementLimitMin", "text",
    "conf.__widget__.YDisplacementLimitMax", "text",
    "conf.__widget__.YDisplacementLimitMin", "text",
    "conf.__widget__.cycleCount", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
IntegratedControlRTC::IntegratedControlRTC(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_displacementIn("displacement", m_displacement),
    m_digitalInputIn("digitalInput", m_digitalInput),
    m_digitalOutputOut("digitalOutput", m_digitalOutput),
    m_ManipulatorCommonInterface_CommonPort("ManipulatorCommonInterface_Common"),
    m_ManipulatorCommonInterface_MiddlePort("ManipulatorCommonInterface_Middle")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
IntegratedControlRTC::~IntegratedControlRTC()
{
}



RTC::ReturnCode_t IntegratedControlRTC::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("displacement", m_displacementIn);
  addInPort("digitalInput", m_digitalInputIn);
  
  // Set OutPort buffer
  addOutPort("digitalOutput", m_digitalOutputOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_ManipulatorCommonInterface_CommonPort.registerConsumer("ManipulatorCommonInterface_Common", "JARA_ARM::ManipulatorCommonInterface_Common", m_ManipulatorCommonInterface_Common);
  m_ManipulatorCommonInterface_MiddlePort.registerConsumer("ManipulatorCommonInterface_Middle", "JARA_ARM::ManipulatorCommonInterface_Middle", m_ManipulatorCommonInterface_Middle);
  
  // Set CORBA Service Ports
  addPort(m_ManipulatorCommonInterface_CommonPort);
  addPort(m_ManipulatorCommonInterface_MiddlePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("teachingData", m_teachingData, "PointData.txt");
  bindParameter("XDisplacementLimitMax", m_XDisplacementLimitMax, "50.0");
  bindParameter("XDisplacementLimitMin", m_XDisplacementLimitMin, "-100.0");
  bindParameter("YDisplacementLimitMax", m_YDisplacementLimitMax, "10");
  bindParameter("YDisplacementLimitMin", m_YDisplacementLimitMin, "-10");
  bindParameter("cycleCount", m_cycleCount, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t IntegratedControlRTC::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t IntegratedControlRTC::onActivated(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"           Activate           "<<std::endl<<"******************************"<<std::endl<<std::endl;

	Sleep(5000);

	//eB[`Of[^ǂݍ
	FILE * fp;
	int len = m_teachingData.length();
	char* FileName = new char[len+1];
	memcpy(FileName,m_teachingData.c_str(),len+1);
	
	fp=fopen( FileName , "r");
	if( fp == NULL ) {
		std::cout<<"t@Cǂݍ߂܂"<<std::endl;
		RTC_ERROR(("t@Cǂݍ߂܂"));
		//ErrorJ
		return RTC::RTC_ERROR;
	}
	
	int i,num;
	for(i=1;i<6;i++){
		fscanf( fp, "P%d=(%lf,%lf,%lf,%lf,%lf,%lf)(%lf,%lf)\n", &num, &m_P[i][0], &m_P[i][1], &m_P[i][2], &m_P[i][3], &m_P[i][4], &m_P[i][5], &m_P[i][6], &m_P[i][7]);	
		std::cout<<"P"<<num<<"=("<<m_P[i][0]<<" , "<<m_P[i][1]<<" , "<<m_P[i][2]<<" , "<<m_P[i][3]<<" , "<<m_P[i][4]<<" , "<<m_P[i][5]<<")("<<m_P[i][6]<<","<<m_P[i][7]<<")"<<std::endl;
		RTC_INFO(("P%d=( %lf , %lf , %lf , %lf , %lf , %lf)( %lf , %lf)",num,m_P[i][0],m_P[i][1],m_P[i][2],m_P[i][3],m_P[i][4],m_P[i][5],m_P[i][6],m_P[i][7]));
		m_P[i][3]=m_P[i][3]/180.0*M_PI;
		m_P[i][4]=m_P[i][4]/180.0*M_PI;
		m_P[i][5]=m_P[i][5]/180.0*M_PI;
		//ϊsɑ
		//pX->Y->Z]ł̒l
			//1
			m_Pos[i].carPos[0][0]=cos(m_P[i][5])*cos(m_P[i][4]);
			m_Pos[i].carPos[1][0]=sin(m_P[i][5])*cos(m_P[i][4]);
			m_Pos[i].carPos[2][0]=-sin(m_P[i][4]);
			
			//2
			m_Pos[i].carPos[0][1]=-sin(m_P[i][5])*cos(m_P[i][3])+cos(m_P[i][5])*sin(m_P[i][4])*sin(m_P[i][3]);
			m_Pos[i].carPos[1][1]=cos(m_P[i][5])*cos(m_P[i][3])+sin(m_P[i][5])*sin(m_P[i][4])*sin(m_P[i][3]);
			m_Pos[i].carPos[2][1]=cos(m_P[i][4])*sin(m_P[i][3]);
			
			//3
			m_Pos[i].carPos[0][2]=sin(m_P[i][5])*sin(m_P[i][3])+cos(m_P[i][5])*sin(m_P[i][4])*cos(m_P[i][3]);
			m_Pos[i].carPos[1][2]=-cos(m_P[i][5])*sin(m_P[i][3])+sin(m_P[i][5])*sin(m_P[i][4])*cos(m_P[i][3]);
			m_Pos[i].carPos[2][2]=cos(m_P[i][4])*cos(m_P[i][3]);
			
			//4
#ifdef RECTANGULAR
			m_Pos[i].carPos[0][3]=(double)m_P[i][0];//[mm]
			m_Pos[i].carPos[1][3]=(double)m_P[i][1];//[mm]
			m_Pos[i].carPos[2][3]=(double)m_P[i][2];//[mm]
#endif
#ifdef VERTICAL
			m_Pos[i].carPos[0][3]=(double)m_P[i][0]/1000.0;//[mm]->[m]
			m_Pos[i].carPos[1][3]=(double)m_P[i][1]/1000.0;//[mm]->[m]
			m_Pos[i].carPos[2][3]=(double)m_P[i][2]/1000.0;//[mm]->[m]
#endif
	
	}
	//t@CN[Y
	fclose( fp );
	
	//JԂ񐔏
	m_Repeat = 0;

	//DIO
	m_digitalOutput.data.length(3);
	m_digitalOutput.data[0]=false;
	m_digitalOutput.data[1]=false;
	m_digitalOutput.data[2]=false;
	m_digitalOutputOut.write();

	//\tg~bglݒ
	std::cout<<"\tg~bgln[h~bglɐݒ"<<std::endl;
	RTC_INFO(("\tg~bgln[h~bglɐݒ"));
	JARA_ARM::LimitSeq softLimit;
#ifdef RECTANGULAR
	softLimit.length(2);
	softLimit[0].upper=0.20;//[m]
	softLimit[0].lower=0.00;//[m]
	softLimit[1].upper=0.10;//[m]
	softLimit[1].lower=0.0;//[m]
#endif
#ifdef VERTICAL
	softLimit.length(6);
	softLimit[0].upper=170.0/180.0*M_PI;//[rad]
	softLimit[0].lower=-170.0/180.0*M_PI;//[rad]
	softLimit[1].upper=135.0/180.0*M_PI;//[rad]
	softLimit[1].lower=-90.0/180.0*M_PI;//[rad]
	softLimit[2].upper=171.0/180.0*M_PI;//[rad]
	softLimit[2].lower=-20.0/180.0*M_PI;//[rad]
	softLimit[3].upper=160.0/180.0*M_PI;//[rad]
	softLimit[3].lower=-160.0/180.0*M_PI;//[rad]
	softLimit[4].upper=120.0/180.0*M_PI;//[rad]
	softLimit[4].lower=-120.0/180.0*M_PI;//[rad]
	softLimit[5].upper=360.0/180.0*M_PI;//[rad]
	softLimit[5].lower=-360.0/180.0*M_PI;//[rad]
#endif
	m_rid=m_ManipulatorCommonInterface_Common->setSoftLimitJoint(softLimit);
	if(m_rid->id != 0){//Error
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}
	
	//x[%]ݒ
	std::cout<<"ő呬xɑ΂銄30[%]ɐݒ"<<std::endl;
	RTC_INFO(("ő呬xɑ΂銄30[%]ɐݒ"));
#ifdef RECTANGULAR
	m_rid=m_ManipulatorCommonInterface_Middle->setSpeedCartesian(30);
#endif
#ifdef VERTICAL
	JARA_ARM::CartesianSpeed Cspeed;
	Cspeed.rotation = 30.0;
	Cspeed.translation = 30.0;
	m_rid=m_ManipulatorCommonInterface_Middle->setMaxSpeedCartesian(Cspeed);
#endif
	if(m_rid->id != 0){//Error
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}
#ifdef RECTANGULAR
	m_rid=m_ManipulatorCommonInterface_Middle->setSpeedJoint(30);
	if(m_rid->id != 0){//Error
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}
#endif
	

	//JnmF(y/n)
	char c;
#ifdef RECTANGULAR
	std::cout<<"_Aʒu(J1=0.0[mm],J2=0.0[mm])ɈړJn܂(y/n)"<<std::endl;
	std::cout<<"n̏ꍇRTCERRORԂɑJڂ܂"<<std::endl;
	RTC_INFO(("_Aʒu(J1=0.0[mm],J2=0.0[mm])ɈړJn܂(y/n)"));
	RTC_INFO(("n̏ꍇRTCERRORԂɑJڂ܂"));
#endif
#ifdef VERTICAL
	std::cout<<"_Aʒu(J1=0[deg],J2=0[deg],J3=90[deg],J4=0[deg],J5=90[deg],J6=0[deg])ɈړJn܂(y/n)"<<std::endl;
	std::cout<<"n̏ꍇRTCERRORԂɑJڂ܂"<<std::endl;
	RTC_INFO(("_Aʒu(J1=0[deg],J2=0[deg],J3=90[deg],J4=0[deg],J5=90[deg],J6=0[deg])ɈړJn܂(y/n)"));
	RTC_INFO(("n̏ꍇRTCERRORԂɑJڂ܂"));
#endif
	std::cin>>c;
	switch(c){
	case 'y':
		//T[{ON
		m_rid=m_ManipulatorCommonInterface_Common->servoON();
		if(m_rid->id != 0){//Error
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		break;
	default:
		//ErrorJ
		return RTC::RTC_ERROR;
	}

#ifdef RECTANGULAR
	//nhJ(DIOo̓|[g[1],[2])
	std::cout<<"nhOPEN"<<std::endl;
	RTC_INFO(("nhOPEN"));
	m_digitalOutput.data[1]=true;
	m_digitalOutput.data[2]=false;
	m_digitalOutputOut.write();
	while(1){
		if(m_digitalInputIn.isNew()){
			m_digitalInputIn.read();
			if(m_digitalInput.data[1] == true){
				break;
			}
		}
	}
#endif
#ifdef VERTICAL
	//nhOFF(DIOo̓|[g[1])
	std::cout<<"nhOFF"<<std::endl;
	RTC_INFO(("nhOFF"));
	m_digitalOutput.data[1]=false;
	m_digitalOutputOut.write();
#endif

	//_Aʒu(PHome)Ɉړ
	JARA_ARM::JointPos jnt;
#ifdef RECTANGULAR
	jnt.length(2);		
	jnt[0]=0.0;//[m]
	jnt[1]=0.0;//[m]
#endif
#ifdef VERTICAL
	jnt.length(6);		
	jnt[0]=0.0/180.0*M_PI;//[rad]
	jnt[1]=0.0/180.0*M_PI;//[rad]
	jnt[2]=90.0/180.0*M_PI;//[rad]
	jnt[3]=0.0/180.0*M_PI;//[rad]
	jnt[4]=90.0/180.0*M_PI;//[rad]
	jnt[5]=180.0/180.0*M_PI;//[rad]
#endif
	m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}

	do{
		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_ERROR(("A["));
			return RTC::RTC_ERROR;
		}
	}while(m_state&0x2);

	std::cout<<std::endl<<"******************************"<<std::endl<<"           Execute            "<<std::endl<<"******************************"<<std::endl<<std::endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t IntegratedControlRTC::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"          Deactivate          "<<std::endl<<"******************************"<<std::endl<<std::endl;

	//DIO
	m_digitalOutput.data.length(3);
	m_digitalOutput.data[0]=false;
	m_digitalOutput.data[1]=false;
	m_digitalOutput.data[2]=false;
	m_digitalOutputOut.write();

	do{
		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_ERROR(("A["));
			return RTC::RTC_ERROR;
		}
	}while(m_state&0x2);

	//_Aʒu(PHome)Ɉړ
	JARA_ARM::JointPos jnt;
#ifdef RECTANGULAR
	jnt.length(2);		
	jnt[0]=0.0;//[m]
	jnt[1]=0.0;//[m]
#endif
#ifdef VERTICAL
	jnt.length(6);		
	jnt[0]=0.0/180.0*M_PI;//[rad]
	jnt[1]=0.0/180.0*M_PI;//[rad]
	jnt[2]=90.0/180.0*M_PI;//[rad]
	jnt[3]=0.0/180.0*M_PI;//[rad]
	jnt[4]=90.0/180.0*M_PI;//[rad]
	jnt[5]=0.0/180.0*M_PI;//[rad]
#endif
	m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		
		JARA_ARM::AlarmSeq_var alarms;
		m_ManipulatorCommonInterface_Common->getActiveAlarm(alarms);
		int len=alarms->length();
		if(len == 0){
			std::cout << "Non Alarm"<< std::endl;
			RTC_INFO(("Non Alarm"));
		}else{
			for(int i=0;i<len;i++){
				std::cout << "Alarm"<< std::endl << alarms[i].description << std::endl;
				RTC_INFO(("Alarm : %s",alarms[i].description));
			}
		}

		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x1){
			std::cout <<"T[{ON"<< std::endl;
			RTC_INFO(("T[{ON"));
		}
		if(m_state&0x2){
			std::cout <<"쒆"<< std::endl;
			RTC_INFO(("쒆"));
		}
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_INFO(("A["));
		}
		if(m_state&0x8){
			std::cout <<"Move߂̃obt@t"<< std::endl;
			RTC_INFO(("Move߂̃obt@t"));
		}
		if(m_state&0x10){
			std::cout <<"ꎞ~"<< std::endl;
			RTC_INFO(("ꎞ~"));
		}

		//ErrorJ
		return RTC::RTC_ERROR;
	}

	do{
		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_ERROR(("A["));
			return RTC::RTC_ERROR;
		}
	}while(m_state&0x2);

	//T[{OFF
	std::cout<<"T[{OFF"<<std::endl;
	RTC_INFO(("T[{OFF"));
	m_rid=m_ManipulatorCommonInterface_Common->servoOFF();
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}

  return RTC::RTC_OK;
}


RTC::ReturnCode_t IntegratedControlRTC::onExecute(RTC::UniqueId ec_id)
{
	if(m_Repeat >= m_cycleCount){
		m_cycleCount = 0;
		m_Repeat = 0;
		Sleep(200);
	}else{
		m_Repeat++;
		std::cout<<"JԂ:"<<m_Repeat<<""<<std::endl<<"(c"<<m_cycleCount-m_Repeat<<")"<<std::endl;
		RTC_INFO(("JԂ:%d\n(c%d)",m_Repeat,m_cycleCount-m_Repeat));


		double XMin = m_XDisplacementLimitMin;
		int count=0;

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P1,Mov)
#ifdef RECTANGULAR
		std::cout<<"{bg(P1,JMov)"<<std::endl;
		RTC_INFO(("{bg(P1,JMov)"));
		JARA_ARM::JointPos jnt;
		jnt.length(2);		
		jnt[0]=m_Pos[1].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[1].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P1,Mov)"<<std::endl;
		RTC_INFO(("{bg(P1,Mov)"));
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPCartesianAbs(m_Pos[1]);
#endif
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);
		
		//RxA(DIOo̓|[g[0])
		m_digitalOutput.data[0]=true;
		m_digitalOutputOut.write();
		
		//[v
		while(1){
			//f[^|[g烏[Nʒu̎擾
			if(m_displacementIn.isNew()){
				m_displacementIn.read();
			
				//
				if((m_displacement.data.x*1000.0>XMin)&&(m_displacement.data.x*1000.0<m_XDisplacementLimitMax)&&(m_displacement.data.y*1000.0>m_YDisplacementLimitMin)&&(m_displacement.data.y*1000.0<m_YDisplacementLimitMax)){
					count++;
					XMin = m_displacement.data.x*1000.0;
					if(count == 2){
						//RxA~(DIOo̓|[g[0])
						m_digitalOutput.data[0]=false;
						m_digitalOutputOut.write();
						break;
					}
				
				}else if(m_displacement.data.x != XMin){

					XMin = m_XDisplacementLimitMin;
					count = 0;

					//ԊOZT(DIO̓|[g[0])mF
					if(m_digitalInputIn.isNew()){
						m_digitalInputIn.read();
						if(m_digitalInput.data[0] == true){
							std::cout << "ԊOZTmF" << std::endl;
							std::cout << "xgRxA~AT[{OFF" << std::endl;
							RTC_ERROR(("ԊOZTmF"));
							RTC_ERROR(("xgRxA~AT[{OFF"));

							//~
							m_digitalOutput.data[0]=false;
							m_digitalOutput.data[1]=false;
							m_digitalOutput.data[2]=false;
							m_digitalOutputOut.write();
							m_ManipulatorCommonInterface_Common->servoOFF();
							//ErrorJ
							return RTC::RTC_ERROR;
						}
					}
				}
			}
		}

		count=0;
		double XPosition=0.0 , YPosition=0.0;
		static double XPositionBuf[7],YPositionBuf[7];
		//f[^|[g烏[Nʒu̎擾(3񕽋ϒl)
		while(1){
			if(m_displacementIn.isNew()){
				m_displacementIn.read();
								
				if((m_displacement.data.x*1000.0>m_XDisplacementLimitMin)&&(m_displacement.data.x*1000.0<m_XDisplacementLimitMax)&&(m_displacement.data.y*1000.0>m_YDisplacementLimitMin)&&(m_displacement.data.y*1000.0<m_YDisplacementLimitMax)){
					
					if(count>=4){
						XPosition = XPosition+m_displacement.data.x;
						YPosition = YPosition+m_displacement.data.y;
					}
					XPositionBuf[count]=m_displacement.data.x;
					YPositionBuf[count]=m_displacement.data.y;

					count++;
					if(count == 7){
						XPosition = XPosition/3.0;
						YPosition = YPosition/3.0;
						for(int i=4;i<7;i++){
							if(fabs(XPositionBuf[i]-XPosition) > 0.02){//[m]
								std::cout<<"[NX΍ERROR"<<std::endl;
								RTC_ERROR(("[NX΍ERROR"));
								return RTC::RTC_ERROR;
							}else if(fabs(YPositionBuf[i]-YPosition) > 0.02){//[m]
								std::cout<<"[NY΍ERROR"<<std::endl;
								RTC_ERROR(("[NY΍ERROR"));
								return RTC::RTC_ERROR;
							}
						}
						break;
					}
				}else{
					std::cout<<"[NF͈͊OERROR"<<std::endl;
					RTC_ERROR(("[NF͈͊OERROR"));
					return RTC::RTC_ERROR;
				}
			}
		}

		//[Nʒu̍W
#ifdef RECTANGULAR
		//{bgXW([NxW)
		m_Pos[2].carPos[0][3]=m_P[2][0]+XPosition*1000.0;//[mm]
		m_Pos[3].carPos[0][3]=m_P[3][0]+XPosition*1000.0;//[mm]
#endif
#ifdef VERTICAL
		//{bgYW([NxW)
		m_Pos[2].carPos[1][3]=m_P[2][1]/1000.0+XPosition;//[m]
		m_Pos[3].carPos[1][3]=m_P[3][1]/1000.0+XPosition;//[m]
		//{bgXW([NYW)
		m_Pos[2].carPos[0][3]=m_P[2][0]/1000.0+YPosition;//[m]
		m_Pos[3].carPos[0][3]=m_P[3][0]/1000.0+YPosition;//[m]
#endif

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P2,Mov)
#ifdef RECTANGULAR
		std::cout<<"{bg(P2,JMov)"<<std::endl;
		RTC_INFO(("{bg(P2,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[2].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[2].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P2,Mvs)"<<std::endl;
		RTC_INFO(("{bg(P2,Mvs)"));
		m_rid=m_ManipulatorCommonInterface_Middle->moveLinearCartesianAbs(m_Pos[2]);
#endif
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P3,Mvs)
#ifdef RECTANGULAR
		std::cout<<"{bg(P3,JMov)"<<std::endl;
		RTC_INFO(("{bg(P3,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[3].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[3].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P3,Mvs)"<<std::endl;
		RTC_INFO(("{bg(P3,Mvs)"));
		m_rid=m_ManipulatorCommonInterface_Middle->moveLinearCartesianAbs(m_Pos[3]);
#endif
		
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);


#ifdef RECTANGULAR
		//nh(DIOo̓|[g[1],[2])
		std::cout<<"nhCLOSE"<<std::endl;
		RTC_INFO(("nhCLOSE"));
		m_digitalOutput.data.length(3);
		m_digitalOutput.data[1]=false;
		m_digitalOutput.data[2]=true;
		m_digitalOutputOut.write();
		while(1){
			if(m_digitalInputIn.isNew()){
				m_digitalInputIn.read();
				if(m_digitalInput.data[1] == true){
					break;
				}
			}
		}
#endif
#ifdef VERTICAL
		//nhON(DIOo̓|[g[1])
		std::cout<<"nhON"<<std::endl;
		RTC_INFO(("nhON"));
		m_digitalOutput.data.length(3);
		m_digitalOutput.data[1]=true;
		m_digitalOutputOut.write();
		Sleep(500);//Ŝ
#endif
		
		//{bg(P2,Mvs)
#ifdef RECTANGULAR
		std::cout<<"{bg(P2,JMov)"<<std::endl;
		RTC_INFO(("{bg(P2,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[2].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[2].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P2,Mvs)"<<std::endl;
		RTC_INFO(("{bg(P2,Mvs)"));
		m_rid=m_ManipulatorCommonInterface_Middle->moveLinearCartesianAbs(m_Pos[2]);
#endif

		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P4,Mvs)
#ifdef RECTANGULAR
		std::cout<<"{bg(P4,JMov)"<<std::endl;
		RTC_INFO(("{bg(P4,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[4].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[4].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P4,Mvs)"<<std::endl;
		RTC_INFO(("{bg(P4,Mvs)"));
		m_rid=m_ManipulatorCommonInterface_Middle->moveLinearCartesianAbs(m_Pos[4]);
#endif
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P5,Mvs)
#ifdef RECTANGULAR
		std::cout<<"{bg(P5,JMov)"<<std::endl;
		RTC_INFO(("{bg(P5,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[5].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[5].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P5,Mvs)"<<std::endl;
		RTC_INFO(("{bg(P5,Mvs)"));
		m_rid=m_ManipulatorCommonInterface_Middle->moveLinearCartesianAbs(m_Pos[5]);
#endif
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

#ifdef RECTANGULAR
		//nhJ(DIOo̓|[g[1],[2])
		std::cout<<"nhOPEN"<<std::endl;
		RTC_INFO(("nhOPEN"));
		m_digitalOutput.data.length(3);
		m_digitalOutput.data[1]=true;
		m_digitalOutput.data[2]=false;
		m_digitalOutputOut.write();
		while(1){
			if(m_digitalInputIn.isNew()){
				m_digitalInputIn.read();
				if(m_digitalInput.data[1] == true){
					break;
				}
			}
		}
#endif
#ifdef VERTICAL
		//nhOFF(DIOo̓|[g[1])
		std::cout<<"nhOFF"<<std::endl;
		RTC_INFO(("nhOFF"));
		m_digitalOutput.data.length(3);
		m_digitalOutput.data[1]=false;
		m_digitalOutputOut.write();
		Sleep(500);//Ŝ
#endif

		//{bg(P4,Mvs)
#ifdef RECTANGULAR
		std::cout<<"{bg(P4,JMov)"<<std::endl;
		RTC_INFO(("{bg(P4,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[4].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[4].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P4,Mvs)"<<std::endl;
		RTC_INFO(("{bg(P4,Mvs)"));
		m_rid=m_ManipulatorCommonInterface_Middle->moveLinearCartesianAbs(m_Pos[4]);
#endif
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P1,Mov)
#ifdef RECTANGULAR
		std::cout<<"{bg(P1,JMov)"<<std::endl;
		RTC_INFO(("{bg(P1,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[1].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[1].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
#endif
#ifdef VERTICAL
		std::cout<<"{bg(P1,Mov)"<<std::endl;
		RTC_INFO(("{bg(P1,Mov)"));
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPCartesianAbs(m_Pos[1]);
#endif
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

	}

  return RTC::RTC_OK;
}


RTC::ReturnCode_t IntegratedControlRTC::onAborting(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"            Error             "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t IntegratedControlRTC::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t IntegratedControlRTC::onReset(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"            Reset             "<<std::endl<<"******************************"<<std::endl<<std::endl;

	std::cout<<"A[NA܂"<<std::endl;
	RTC_INFO(("A[NA܂"));

	//A[NA
	m_rid=m_ManipulatorCommonInterface_Common->clearAlarms();
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t IntegratedControlRTC::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void IntegratedControlRTCInit(RTC::Manager* manager)
  {
    coil::Properties profile(integratedcontrolrtc_spec);
    manager->registerFactory(profile,
                             RTC::Create<IntegratedControlRTC>,
                             RTC::Delete<IntegratedControlRTC>);
  }
  
};


